/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common"
import "../common/ClockUtils.js" as Utils

ClockBase {
    id: alarmFace

    property alias showSetupMessage: _setupMessage.visible
    property alias currentTimeLabel: _currentTimeLabel
    property alias nextAlarmText: _nextAlarmText
    property string timeFormat: appSetting.contents.timeFormat

    showSecondHand: false
    enableMouseArea: _setupMessage.visible

    innerLabel.visible: !_setupMessage.visible
    innerLabel.font.pixelSize: timeFormat === "12-hour" ? units.dp(31): units.dp(41)

    Image {
        id: _alarmHintImage
        smooth: true
        width: units.gu(40)
        fillMode: Image.PreserveAspectFit
        source: Qt.resolvedUrl("../images/alarm_black.png")
        anchors.centerIn: parent
    }

    Label {
        id: _nextAlarmText

        text: i18n.tr("Next Alarm")
        fontSize: "small"
        parent: alarmFace.centerItem
        visible: !_setupMessage.visible
        width: parent.width; height: width;
        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter
        wrapMode: Text.WordWrap
        anchors {
            centerIn: parent
            verticalCenterOffset: units.gu(-3.5)
        }
    }

    Label {
        id: _currentTimeLabel

        text: i18n.tr("Now")
        fontSize: "small"
        parent: alarmFace.centerItem
        visible: !_setupMessage.visible
        anchors {
            centerIn: parent
            verticalCenterOffset: units.gu(4.5)
        }
    }

    Label {
        id: _setupMessage
        objectName: "labelAlarmSetup"

        parent: alarmFace.centerItem
        width: parent.width; height: width;
        anchors.centerIn: parent
        horizontalAlignment: Text.AlignHCenter
        verticalAlignment: Text.AlignVCenter
        color: Theme.palette.normal.baseText

        fontSize: "large"
        text: i18n.tr("Long press to setup an alarm")
        wrapMode: Text.WordWrap
    }
}
