/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0

// XML Model to retrieve the timezone info of a city searched by the user online.
XmlListModel {
    id: cityTimezoneModel;

    readonly property string timezone_base_url: "http://api.geonames.org/timezone?lat=";

    // TODO: Change username to something along the lines of "com.ubuntu.clock"
    readonly property string username: "krnekhelesh";

    // Function to return the online url for timezone info of a city
    function getCityTimezoneUrl (lat, lng) {
        return (timezone_base_url + lat + "&lng=" + lng + "&username=" + username);
    }

    // Function to calculate the time difference w.r.t UTC
    function getTimeDifference(data) {
        var worldTime = data.split(' ')[1].split(':');
        var hoursWT = parseInt(worldTime[0]);
        var minutesWT = parseInt(worldTime[1]);
        var now = new Date();
        var UTCdiff = new Date().getTimezoneOffset()
        now.setMinutes(now.getMinutes() + UTCdiff)

        // Calculating world city time diff w.r.t UTC
        return ((hoursWT - now.getHours()) * 60 + (minutesWT - now.getMinutes()));
    }

    query: "/geonames/timezone"
    XmlRole { name: "time"; query: "time/string()" }
    onSourceChanged: reload();
}
