/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0
import "../common/ClockUtils.js" as Utils

// Qml Element to draw the easter egg (sunrise and sunset) on the analogue clock face.
Rectangle {
    id: easterEggCircle;

    // FIXME: Replace these constant values with user's location coordinates when automatic location detection is implemented.
    property real latitude: 200;
    property real longitude: 200;

    // properties to set/retrieve the sunrise and sunset times
    property alias sunriseLabel: sunriseTimeLabel.text;
    property alias sunsetLabel: sunsetTimeLabel.text;

    // properties to get the sunRiseModel XML model.
    property alias isReady: sunRiseModel.status;
    property alias model: sunRiseModel;    

    opacity: 0
    color: "transparent"
    z: parent.z + 100
    visible: opacity == 0 ? false : true

    // Function to convert online date format from "2013-04-20 19:40" to just "19:40"
    function getSunTime (data) {
        var sunTime = data.split(' ')[1]
        return Utils.convertTime(sunTime.split(":")[0], sunTime.split(":")[1], 0, appSetting.contents.timeFormat)
    }

    onLatitudeChanged: sunRiseModel.source = sunRiseModel.getUrlString(latitude, longitude)
    onLongitudeChanged: sunRiseModel.source = sunRiseModel.getUrlString(latitude, longitude)

    // U1dB Document to cache the sunrise/sunset times
    EasterEggStorage {
        id: cache
    }

    // TODO 1: Get the latitude and longitude using the platform API automatically instead of using static values.    
    EasterEggModel {
        id: sunRiseModel
        onStatusChanged: {
            var checkedDate = cache.contents.cacheTime.split('T')[0]
            var date = new Date().toISOString();
            if (status === XmlListModel.Ready && checkedDate !== date.split('T')[0] && longitude !== 200 && latitude !== 200){
                Utils.log("Retrieving Sunrise/Sunset times fron online API")
                sunriseTimeLabel.text = getSunTime(sunRiseModel.get(0).sunriseTime);
                sunsetTimeLabel.text = getSunTime(sunRiseModel.get(0).sunsetTime);
                cache.updateCache(date, sunriseTimeLabel.text, sunsetTimeLabel.text);
            }
            else if (status === XmlListModel.Ready) {
                Utils.log("Retrieving Sunrise/Sunset times from disk");
                sunriseTimeLabel.text = cache.contents.riseTime;
                sunsetTimeLabel.text = cache.contents.setTime;
            }
        }
    }
    
    Column {        
        anchors.centerIn: parent
        width: parent.width; height: childrenRect.height
        
        Label {
            id: sunriseLabel
            anchors.horizontalCenter: parent.horizontalCenter
            color: Theme.palette.normal.baseText
            fontSize: "medium"
            text: i18n.tr("Sunrise")
        }
        
        Label {
            id: sunriseTimeLabel
            anchors.horizontalCenter: parent.horizontalCenter
            color: Theme.palette.normal.baseText
            font.pixelSize: appSetting.contents.timeFormat === "12-hour" ? units.dp(28) : units.dp(38)
            text: i18n.tr("Loading")
        }
        
        Rectangle {
            id: sunsetsunriseDivider
            width: parent.width - units.gu(2)
            height: units.gu(0.1)
            anchors.horizontalCenter: parent.horizontalCenter
            color: Theme.palette.normal.baseText
        }
        
        Label {
            id: sunsetTimeLabel
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            color: Theme.palette.normal.baseText
            font.pixelSize: sunriseTimeLabel.font.pixelSize
            text: i18n.tr("Loading")
        }
        
        Label {
            id: sunsetLabel
            width: parent.width
            horizontalAlignment: Text.AlignHCenter
            color: Theme.palette.normal.baseText
            fontSize: "medium"
            text: i18n.tr("Sunset")
        }
        
    }
}
