/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0

// Search Box element for world clock. Instant search as you type after a small pause.
Item {
    id: searchBox;

    // Property to hold the string inputted by the user. This is required in other places such as the XML Model for initiating an online query.
    property alias search_string: searchLabel.text

    // Property to store the focus property of the search field
    property bool setFocus: false

    // Signal connected to the search_timer triggered signal to expose it to outside objects
    signal searchTriggered()

    Component.onCompleted: {
        search_timer.triggered.connect(searchTriggered)
        if (setFocus) 
            searchLabel.forceActiveFocus()
    }

    // Search Label
    TextField {
        id: searchLabel

        anchors { left: parent.left; leftMargin: units.gu(2); top: parent.top; bottom: parent.bottom; right: parent.right; rightMargin: units.gu(2) }
        width: parent.width/1.5
        hasClearButton: true
        placeholderText: i18n.tr("Search")
        primaryItem: Image {
            height: parent.height/1.5;
            fillMode: Image.PreserveAspectFit
            source: Qt.resolvedUrl("../images/search_icon.svg")
        }
        
        // Provide a small pause before going online to search
        Timer {
            id: search_timer
            interval: 1100
            repeat: false
        }
        
        onTextChanged: search_timer.restart()
    }
    
    // Indicator to show search activity
    ActivityIndicator {
        id: searchActivity
        anchors { verticalCenter: searchLabel.verticalCenter; right: searchLabel.right; rightMargin: units.gu(1) }
        running: searchCityModel.status === XmlListModel.Loading
    }
}
