/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0

// XML Model to list cities provided locally by clock-app or search online (geoname.org) for more cities and return latitude and longitude information of the city
XmlListModel {
    id: worldCityModel;

    readonly property string search_base_url: "http://api.geonames.org/search?q="
    readonly property string local_base_url: "common-cities.xml"
    readonly property int no_of_results: 5
    property string search_string: ""

    // TODO: Change username to something along the lines of "com.ubuntu.clock"
    readonly property string username: "krnekhelesh";

    // Function to return the online search url for city search
    function searchCityUrl(city) {
        return (search_base_url + search_string + "&maxRows=" + no_of_results + "&username=" + username + "&style=full&featureClass=P&name_startsWith=" + city);
    }

    // Function to return the url of the locally provided city list (xml file)
    function localCityUrl() {
        return Qt.resolvedUrl(local_base_url);
    }
    
    // By default list the cities provided locally by clock app
    source: localCityUrl()
    query: "/geonames/geoname"
    
    XmlRole { name: "city"; query: "toponymName/string()"; isKey: true }
    XmlRole { name: "country"; query: "countryName/string()"; isKey: true }
    XmlRole { name: "lat"; query: "lat/string()"; isKey: true }
    XmlRole { name: "lng"; query: "lng/string()"; isKey: true }
    XmlRole { name: "adminName"; query: "adminName1/string()"; isKey: true}
    XmlRole { name: "adminName2"; query: "adminName2/string()"; isKey: true}
    
    onSourceChanged: reload();
}
