/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Pickers 0.1

/*
  Basic component to draw the analogue clock with hour, minute and second hands.
 */
Dialer {
    id: clockRoot

    // Properties to set the clock hands position
    property alias hours: _hourHand.value
    property alias minutes: _minuteHand.value
    property alias seconds: _secondHand.value

    // Property to set the label shown in the inner circle
    property alias innerLabel: _currentTimeLabel

    // Property to individually hide the hour and minute hand in timer and alarm
    property bool showHourHand: true
    property bool showMinuteHand: true
    property bool showSecondHand: true

    // Property to enable/disable the mouse area of the inner circle.
    property bool enableMouseArea: true

    // Property to set if the clock hands are adjustable by the user
    property bool draggable: false

    // Property to help with incrementing/decrementing hour and changing AM/PM when passing 60
    property int valueWas: 0

    signal clicked(var mouse)
    signal pressAndHold(var mouse)

    minimumValue: 0
    maximumValue: 60
    size: units.gu(32)

    DialerHand {
        id: _hourHand
        objectName: "hourHand"
        z: hand.visible ? _minuteHand.z + 1 : -1
        hand.draggable: draggable
        hand.visible: showHourHand
    }

    DialerHand {
        id: _minuteHand
        objectName: "minuteHand"
        hand.draggable: draggable
        hand.visible: showMinuteHand
    }

    DialerHand {
        id: _secondHand
        objectName: "secondHand"
        hand.draggable: draggable
        hand.visible: showSecondHand
    }

    centerContent: [
        Label {
            id: _currentTimeLabel
            objectName: "label"
            anchors.centerIn: parent
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            font.pixelSize: units.dp(41)
        },

        MouseArea {
            anchors.fill: parent
            enabled: enableMouseArea
            onClicked: clockRoot.clicked(mouse);
            onPressAndHold: clockRoot.pressAndHold(mouse);
        }
    ]
}
