/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

/*
  This file lists the keyboard shortcuts function used throughtout the application.

  --- Global Shortcuts ---
  CTRL + TAB         : Navigate to the next tab
  CTRL + SHIFT + TAB : Navigate to the previous tab

  --- Clock Tab ---
  CTRL + N           : Add a world city
  ESCAPE             : Close the world city search page

  --- Alarm Tab ---
  CTRL + N           : Create a new alarm
  ESCAPE             : Close Add Alarm Page

  --- Timer Tab ---
  CTRL + N           : Create a new timer
  ESCAPE             : Cancel creating a timer

  --- Stopwatch Tab ---
  SPACE              : Start/Stop Stopwatch
  ENTER              : Create Stopwatch Lap
  ESCAPE             : Reset Stopwatch
 */

function generalShortcuts(event, maxTabs, selectedTabIndex, pagestack, timerPage, stopwatchPage) {
    if(event.modifiers & Qt.ControlModifier) {
        switch(event.key) {
        case Qt.Key_Tab:
            // Ctrl + Tab to switch to the next tab
            selectedTabIndex = selectedTabIndex === maxTabs-1 ? 0 : selectedTabIndex + 1
            break;

        case Qt.Key_Backtab:
            // Ctrl + Shift + Tab to switch to the previous tab
            selectedTabIndex = selectedTabIndex === 0 ? maxTabs-1 : selectedTabIndex - 1
            break;

        case Qt.Key_N:
            // Ctrl + N to perform appropriate action depending on the current tab
            if(selectedTabIndex === 0)
                pagestack.push(Qt.resolvedUrl("../clock/WorldClock.qml"), {"isWorldCity": true, "setFocus": true})

            else if(selectedTabIndex === 1)
                pagestack.push(Qt.resolvedUrl("../alarm/AddAlarmPage.qml"), {"isNewAlarm": true, "setFocus": true})

            else if(selectedTabIndex === 2) {
                timerPage.setFocus = true
                timerPage.add_preset()
            }

            break;
        }
    }
    event.accepted = false
    return selectedTabIndex
}

function enterShortcut(event, selectedTabIndex, stopwatchPage) {
    // Enter to create a stopwatch lap
    if(selectedTabIndex === 3)
        stopwatchPage.create_lap()

    event.accepted = false
}

function spaceShortcut(event, selectedTabIndex, stopwatchPage) {
    // Space to start/stop the stopwatch
    if(selectedTabIndex === 3)
        stopwatchPage.start_stop_stopwatch()

    event.accepted = false
}

function escapeShortcut(event, selectedTabIndex, pagestack, timerPage, stopwatchPage) {
    // Escape to close the world clock search page
    if(selectedTabIndex === 0 && pagestack.currentPage.objectName === "worldPage")
        pagestack.pop()

    // Escape to cancel saving/updating an alarm
    else if(selectedTabIndex === 1 && pagestack.currentPage.objectName === "addAlarmPage")
        pagestack.pop()

    // Escape to cancel saving timer preset
    else if(selectedTabIndex === 2)
        timerPage.cancel_preset()

    // Escape to reset stopwatch
    else if(selectedTabIndex === 3)
        stopwatchPage.reset_stopwatch()

    event.accepted = false
}
