/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "../common"

// Lap list column to show the laps recorded by the user
Column {
    id: listLap

    // Property to hold component heights required for the dynamic positioning of the column
    property int dynamicTopSpacing
    property alias dividerHeight: divider.height
    property alias headerHeight: locationHeader.height

    // Lap model required by the listview
    property alias lapModel: listViewLap.model

    height: childrenRect.height    

    ListItem.ThinDivider { id: divider}

    ListItem.Header {
        id: locationHeader
        Label {
            text: i18n.tr("Recorded laps")
            anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
            color: Theme.palette.normal.baseText
            fontSize: "medium"
        }
    }

    ListView {
        id: listViewLap

        clip: true
        anchors { left: parent.left; right: parent.right; }
        height: 5 * units.gu(6)
        model: lapModel

        delegate: ListItem.Standard {
            objectName: "laps" + index
            Label {
                text: Utils.numberToLocaleString(lapModel.count - index);
                anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                color: Theme.palette.normal.baseText
                fontSize: "large"
            }

            removable: true
            // FIXME: Confirm removal for stopwatch is not required. However if set to false, then the SDK does not display
            // the delete text and icon leading to the user not being aware which action is performed on swiping left/right.
            // Reported bug at https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1242089
            confirmRemoval: true
            onItemRemoved: lapModel.remove(index)

            Label {
                text: label;
                anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                color: Theme.palette.normal.baseText
                fontSize: "large"
            }
        }
    }
}
