/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "StopwatchSupport.js" as SScript
import "../common"

ClockBase {
    id: stopwatchFace

    // Properties to store/set the time variables
    property var startTime: 0
    property var stopTime: 0
    property int delay : 0
    property bool flag: true
    property int totalTime: 0
    property int milliseconds: 0

    // Sets the location and visible (bool) values of the blip (small circle visible when the lap button is pressed)
    property alias blipLocation: blipFist.location
    property alias blipVisible: blipFist.visible

    // Retrieves the status of the stopwatch timer
    property bool timerStatus: stopwatchUIUpdate.running

    // Property to start the stopwatch hint animation
    property alias stopwatchAnimationHint: _stopwatchAnimationHint

    // Property to indicate the current status of the stopwatch
    property alias stopwatchHintText: _stopwatchHintMessage.text

    innerLabel.font.pixelSize: hours > 0 ? units.dp(28) : units.dp(41)

    function reset() {
        totalTime = milliseconds = seconds = minutes = hours = 0
        SScript.deltaTime[0] = SScript.deltaTime[1] = SScript.deltaTime[2] = SScript.deltaTime[3] = 0
        startTime = stopTime = delay = 0;
        flag = true;
    }

    function start() {
        stopwatchUIUpdate.start()
    }

    function stop() {
        stopwatchUIUpdate.stop()
    }

    // Timer which runs every 100 microseconds to update the time and also the trailing hand gradient.
    Timer {
        id: stopwatchUIUpdate

        interval: 100
        repeat: true
        onTriggered: {
            totalTime = SScript.delta();
            SScript.deltaTime = SScript.msToTime(totalTime)
            hours = SScript.deltaTime[0];
            minutes = SScript.deltaTime[1];
            seconds = SScript.deltaTime[2];
            milliseconds = SScript.deltaTime[3];
        }
    }

    // This component draws the small white circle that is visible everytime the lap button is pressed.
    Blip {
        id: blipFist
    }

    // Image to display a bar over the stopwatch to differentiate it from other tabs
    Image {
        id: _stopwatchHintImage

        smooth: true
        width: units.gu(40)
        anchors.centerIn: parent
        fillMode: Image.PreserveAspectFit
        source: Qt.resolvedUrl("../images/stopwatch_black.png")

        SequentialAnimation on anchors.verticalCenterOffset {
            id: _stopwatchAnimationHint
            UbuntuNumberAnimation { from: units.gu(0); to: units.gu(2) }
            UbuntuNumberAnimation { from: units.gu(2); to: units.gu(0) }
        }
    }

    Label {
        id: _stopwatchHintMessage
        fontSize: "small"
        text: i18n.tr("Start")
        parent: stopwatchFace.centerItem
        anchors {
            centerIn: parent
            verticalCenterOffset: units.gu(4.5)
        }
    }

    Repeater {
        id:stopwatchMarkers

        model: 24

        delegate: AnalogClockMarker {
            number: modelData
            total: 24;
            size: 0.2
            distance: 0.32
        }
    }
}
