/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Renato Araujo Oliveira Filho <renato.filho@canonical.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "StopwatchSupport.js" as SScript
import "../common"

Page {
    id: stopwatchPage

    property alias lapStart: laps.lapStart

    Component.onCompleted: Utils.log("StopwatchPage loaded");

    // Function to create a lap
    function create_lap() {
        if(analogStopwatch.timerStatus === true) {
            laps.push();
            analogStopwatch.blipLocation = analogStopwatch.seconds
            analogStopwatch.blipVisible = true
        }
    }

    // Function to start/stop a stopwatch
    function start_stop_stopwatch() {
        analogStopwatch.stopwatchAnimationHint.start()
        if(analogStopwatch.timerStatus == false) {
            analogStopwatch.stopwatchHintText = i18n.tr("Stop")
            analogStopwatch.start()
            stopWatchLabelUpdate.start()
            if (analogStopwatch.flag) {
                analogStopwatch.startTime = new Date().getTime();
                analogStopwatch.flag = false;
            }
            else {
                analogStopwatch.delay = Math.abs(new Date().getTime() - analogStopwatch.stopTime);
                analogStopwatch.startTime = analogStopwatch.startTime + analogStopwatch.delay;
            }
        } else {
            analogStopwatch.stopwatchHintText = i18n.tr("Start")
            analogStopwatch.stop()
            stopWatchLabelUpdate.stop()
            analogStopwatch.stopTime = new Date().getTime();
        }
    }

    // Function to reset stopwatch
    function reset_stopwatch() {
        analogStopwatch.stopwatchHintText = i18n.tr("Start")
        analogStopwatch.stopwatchAnimationHint.start()
        analogStopwatch.stop()
        analogStopwatch.reset();
        analogStopwatch.innerLabel.text = SScript.timeToString(0, 0, 0, 0);
        lapStart = 0;
        laps.clear();
    }

    // Timer which runs every 100 microseconds to update the stopwatch label
    Timer {
        id: stopWatchLabelUpdate;

        interval: 100;
        repeat: true;
        onTriggered: analogStopwatch.innerLabel.text = SScript.timeToString(analogStopwatch.hours, analogStopwatch.minutes, analogStopwatch.seconds, analogStopwatch.milliseconds)
    }

    ListModel {
        id: laps

        property int lapStart: 0;

        function push() {
            insert(0, {"time" : (analogStopwatch.totalTime - lapStart),
                       "label": SScript.currentLapLabel(analogStopwatch.totalTime, lapStart)})
            lapStart = analogStopwatch.totalTime
        }
    }

    // Items within this container will move up/down depending on the state.
    Flickable {
        id: stopwatchAnimationContainer

        clip: true
        anchors.fill: parent
        contentWidth: parent.width
        contentHeight: analogStopwatch.height + analogStopwatch.anchors.topMargin + listLap.height + listLap.anchors.topMargin + units.gu(3)

        // Component which draws the analog stopwatch face, controls the timer and calculates the time.
        StopwatchFace {
            id: analogStopwatch
            objectName: "stopwatchFace"
            visible: true

            anchors {
                top: parent.top
                topMargin: units.gu(10)
                horizontalCenter: parent.horizontalCenter
            }

            innerLabel.text: SScript.timeToString(hours, minutes, seconds, milliseconds)
            onClicked: start_stop_stopwatch()
        }

        Column {
            visible: analogStopwatch.hours !== 0 || analogStopwatch.minutes !== 0 || analogStopwatch.seconds !== 0 || analogStopwatch.milliseconds !== 0
            anchors {
                horizontalCenter: analogStopwatch.horizontalCenter
                verticalCenter: lapButton.verticalCenter
            }

            Label {
                fontSize: "large"
                anchors.horizontalCenter: parent.horizontalCenter
                text: SScript.currentLapLabel(analogStopwatch.totalTime, laps.lapStart);
            }

            Label {
                text: i18n.tr("Current")
                anchors.horizontalCenter: parent.horizontalCenter
            }
        }

        ImageButton {
            id: resetButton
            objectName: "resetButton"

            anchors { top: analogStopwatch.bottom; topMargin: units.gu(5); right: analogStopwatch.left; rightMargin: units.gu(-1) }
            buttonImage: Qt.resolvedUrl("../images/reset_icon.svg");
            buttonLabel: ""
            onClicked: reset_stopwatch()
        }

        ImageButton {
            id: lapButton
            objectName: "lapButton"

            anchors { top: analogStopwatch.bottom; topMargin: units.gu(5); left: analogStopwatch.right; leftMargin: units.gu(-1) }
            buttonImage: Qt.resolvedUrl("../images/lap_icon.png");
            buttonLabel: ""
            mirrorLayout: true;
            onClicked: create_lap()
        }

        // ListView to display the current lap and recorded lap information.
        LapList {
            id: listLap
            lapModel: laps
            anchors { left: parent.left; right: parent.right; top: analogStopwatch.bottom; topMargin: listLap.dynamicTopSpacing }
            dynamicTopSpacing: stopwatchPage.height - stopwatchPage.header.height - analogStopwatch.height - headerHeight - analogStopwatch.anchors.topMargin - dividerHeight;
        }
    }
}
