# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nicholas Skaggs <nicholas.skaggs@canonical.com>

"""Tests for the Clock App - Alarm"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Is, Not
from autopilot.matchers import Eventually
import time
import datetime
from ubuntu_clock_app.tests import ClockAppTestCase
import unittest


class TestAlarm(ClockAppTestCase):
    """Tests the alarm page features"""

    """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""

    #TODO: alarms shares similarities with stopwatch
    #the utility functions could be shared between them
    #ie, the emulator functions
    def setUp(self):
        super(TestAlarm, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        #move to alarm tab
        self.main_view.switch_to_tab("AlarmTab")

    def select_name_alarm_label(self):
        label = self.main_view.get_alarm_name_label()
        timeout = 0
        while not label.focus and timeout < 10:
            self.pointing_device.click_object(label)
            time.sleep(1)
            timeout += 1
        return label

    @unittest.skip("Pending development, bug 1269064")
    def test_longpress_new_alarm(self):
        """Test longpress on alarm and back button"""
        #TODO: needs this merge
        #https://code.launchpad.net/~elopio/
        #ubuntu-ui-toolkit/fix1239751_go_back/+merge/191002
        #thus is disabled for now until landing

        self.assertThat(self.main_view.get_label_alarm,
                        Eventually(Not(Is(None))))
        self.main_view.longpress_object(self.main_view.get_label_alarm())

        self.assertThat(self.main_view.get_addalarm_page,
                        Eventually(Not(Is(None))))

        toolbar = self.main_view.open_toolbar()

        # Click the add alarm button
        toolbar.go_back()

        self.assertThat(self.main_view.get_alarm_page,
                        Eventually(Not(Is(None))))

    @unittest.skip("Pending development, bug 1269064")
    def test_add_remove_alarm(self):
        """Test to check if button to add and remove an alarm working"""

        #test once
        #use tomorrow as day
        day = datetime.datetime.today().weekday() + 1
        if day > 7:
            day = 0
        self.add_alarm(day, 1)
        self.delete_alarm()

        #test daily
        self.add_alarm(4, 2)
        self.delete_alarm()

        #test weekly
        #self.add_alarm(1, 3)
        #self.delete_alarm()

    def add_alarm(self, day, occurence):
        #setTime = "16:20"

        # Show the toolbar
        toolbar = self.main_view.open_toolbar()

        # Click the add alarm button
        toolbar.click_button("addAlarmButton")

        self.assertThat(lambda: self.main_view.get_alarm_page().select_single(
                        "QQuickFlickable").moving,
                        Eventually(Equals(False)))

        #TODO: these should be broken out for reuse in timer as well
        #as well as allowing for various times
        # Set hour
        self.assertThat(self.main_view.get_alarm_hour_hand,
                        Eventually(Not(Is(None))))
        hour = self.main_view.get_alarm_hour_hand()

        x, y, w, h = hour.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2)
        self.pointing_device.drag(tx, ty - (h / 4), tx - (w / 2), ty + (h / 2))

        self.assertThat(lambda: self.main_view.get_alarm_page().select_single(
                        "QQuickFlickable").moving,
                        Eventually(Equals(False)))

        # Set minute
        self.assertThat(self.main_view.get_alarm_minute_hand,
                        Eventually(Not(Is(None))))
        minute = self.main_view.get_alarm_minute_hand()

        x, y, w, h = minute.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2.5)
        self.pointing_device.drag(tx, ty - (h / 4), tx + (w / 2), ty + (h / 2))

        self.assertThat(lambda: self.main_view.get_alarm_page().select_single(
                        "QQuickFlickable").moving,
                        Eventually(Equals(False)))

        #TODO: Choosing the same day hides the day
        #https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1240629

        self.edit_alarm(day, occurence)


        #TODO: fix time check.. time doesn't always set properly
        #and will randomly fail the test
        #self.assertThat(self.main_view.get_first_alarm_time,
        #                Eventually(Equals(setTime)))

    def edit_alarm(self, day, occurence):
        setName = "test" + str(int(time.time()))
        setStatus = True

        # Write in the label
        self.main_view.drag_page_up("AlarmPage", self)
        self.assertThat(self.main_view.get_alarm_name_label,
                        Eventually(Not(Is(None))))
        label = self.select_name_alarm_label()
        self.assertThat(label.focus, Eventually(Equals(True)))
        self.keyboard.press('Ctrl+a')
        self.assertThat(label.focus, Eventually(Equals(True)))

        #set unique alarm name
        self.keyboard.type(setName)
        self.assertThat(label.text, Eventually(Equals(setName)))

        #set alarm day and frequency
        self.main_view.set_alarm_frequency(occurence, self)

        #depending on occurrence, we set the day differently
        if occurence == 1:
            #one time, use day passed in
            self.main_view.set_alarm_day(day, self)
        elif occurence == 2:
            #daily, ignore day
            pass
        elif occurence == 3:
            #weekly, pass day into dialog
            #TODO: support weekly dialog
            self.main_view.set_alarm_week(day, self)

        #grab current alarm number
        self.assertThat(self.main_view.get_num_of_alarms,
                        Eventually(Not(Is(None))))
        num_of_alarms_old = self.main_view.get_num_of_alarms()

        # Press "Save" toolbar button
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button("saveAlarmButton")

        #assert alarm is added
        self.assertThat(self.main_view.get_num_of_alarms,
                        Eventually(NotEquals(num_of_alarms_old)))

        #check label, time, subtitle, name
        #TODO: decipher proper subtitles and check for them
        #self.assertThat(self.main_view.get_first_alarm_subtitle,
        #                Eventually(Equals( ? )))

        self.assertThat(self.main_view.get_first_alarm_label,
                        Eventually(Not(Is(None))))

        self.assertThat(self.main_view.get_first_alarm_label,
                        Eventually(Equals(setName)))

        self.assertThat(self.main_view.get_first_alarm_status,
                        Eventually(Equals(setStatus)))

    def delete_alarm(self):
        """Test if the swipe of a alarm deletes it"""
        #TODO: fix this with https://code.launchpad.net/~renatofilho/
        #ubuntu-ui-toolkit/fix-1236464/+merge/190496
        #once it lands
        self.assertThat(self.main_view.get_num_of_alarms,
                        Eventually(Not(Is(None))))
        num_of_alarms_old = self.main_view.get_num_of_alarms()

        self.main_view.drag_page_up("AlarmPage", self)

        # Delete the alarm
        self.assertThat(self.main_view.get_first_alarm_list_item,
                        Eventually(Not(Is(None))))
        timeout = 0
        while self.main_view.get_num_of_alarms() != num_of_alarms_old - 1 \
                and timeout < 10:
            first_alarm = self.main_view.get_first_alarm_list_item()
            x, y, w, h = first_alarm.globalRect
            tx = x + (w / 8)
            ty = y + (h / 2)
            self.pointing_device.drag(tx, ty, w - w / 8, ty)
            time.sleep(1)
            timeout += 1

        # Check has been deleted
        self.assertThat(
            self.main_view.get_num_of_alarms,
            Eventually(Equals(num_of_alarms_old - 1)))

    @unittest.skip("Pending development, bug 1269064")
    def test_toggle_and_edit_alarm(self):
        """Tests if a alarm can be toggled on and off and edited"""

        # Create a new alarm
        self.add_alarm(1, 2)

        self.main_view.drag_page_up("AlarmPage", self)

        # Click to select the alarm
        self.assertThat(self.main_view.get_first_alarm_list_item,
                        Eventually(Not(Is(None))))
        first_alarm = self.main_view.get_first_alarm_list_item()

        #grab current values
        self.assertThat(self.main_view.get_first_alarm_label,
                        Eventually(Not(Is(None))))

        oldName = self.main_view.get_first_alarm_label()
        oldStatus = self.main_view.get_first_alarm_status()

        # Perform edit
        self.pointing_device.click_object(first_alarm)

        #TODO: objects aren't quite the same :-(
        #self.edit_alarm(1, 1)
        #just save it instead
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button("saveAlarmButton")

        #verify everything is still good
        self.assertThat(self.main_view.get_first_alarm_label,
                        Eventually(Not(Is(None))))

        self.assertThat(self.main_view.get_first_alarm_label,
                        Eventually(Equals(oldName)))

        self.assertThat(self.main_view.get_first_alarm_status,
                        Eventually(Equals(oldStatus)))

        # Click to switch off the alarm
        self.assertThat(self.main_view.get_first_alarm_status,
                        Eventually(Equals(True)))

        self.main_view.toggle_first_alarm(self)

        self.assertThat(self.main_view.get_first_alarm_status,
                        Eventually(Equals(False)))

        # Click to switch on the alarm
        self.main_view.toggle_first_alarm(self)

        self.assertThat(self.main_view.get_first_alarm_status,
                        Eventually(Equals(True)))

        self.delete_alarm()
