# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Renato Araujo Oliveira Filho <renato@canonical.com>
#              Nekhelesh Ramananthan <krnekhelesh@gmail.com>

"""
Tests for the Clock App, main window.
"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals
from autopilot.matchers import Eventually
from time import sleep

from ubuntu_clock_app.tests import ClockAppTestCase


class TestClock(ClockAppTestCase):
    """Test the clock page features."""

    def setUp(self):
        """This is needed to wait for the application to start.

        In the testfarm, the application may take some time to show up.

        """

        super(TestClock, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestClock, self).tearDown()

    def add_local_world_city(self):
        # Open clock toolbar        
        self.main_view.open_toolbar()

        # Click add city toolbar button
        self.main_view.get_toolbar().click_button('addCityAction')

        # Click on first city in the list shown and ensure that it was saved
        city = self.main_view.grab_first_listed_city()

        city_name = self.main_view.get_city_name(city)
        self.main_view.select_city(city)

        return city_name

    def test_label_value(self):
        """The clock label must contain the correct time."""

        label = self.main_view.get_time_label()

        # TODO: investigate a way to get the system wide format for time
        # and compare the values
        self.assertThat(label.text, Eventually(NotEquals('')))

    def test_add_local_world_city(self):
        """Test to check if adding a local listed world city works"""

        # Add a local world city  
        city_name = self.add_local_world_city()
        
        saved_city_name = self.main_view.get_city_name(self.main_view.grab_first_saved_city())
        self.assertThat(saved_city_name, Eventually(Equals(city_name)))      
        
    def test_delete_local_world_city(self):
        """Test to check if deleting a saved local world city works"""

        # Add a local world city  
        city_name = self.add_local_world_city()

        # Close toolbar and drag page up to see the saved world city list        
        self.main_view.close_toolbar()
        self.main_view.drag_page_up("ClockPage", self)

        # Get the saved world city and swipe to delete
        old_saved_city_count = self.main_view.get_saved_cities_list().count
        saved_city = self.main_view.grab_first_saved_city()
        saved_city.swipe_to_delete()
        saved_city.confirm_removal()

        # Assert deletion of saved city
        cities = self.main_view.get_saved_cities_list()
        self.assertThat(
            cities.count,
            Eventually(Equals(old_saved_city_count - 1)))
