# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Riccardo Padovani <rpadovani@ubuntu-it.org>
#              Nekhelesh Ramananthan <krnekhelesh@gmail.com>

"""Tests for the Clock App - Timer"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Is, Not
from autopilot.matchers import Eventually
from time import sleep

from ubuntu_clock_app.tests import ClockAppTestCase
import logging

logger = logging.getLogger(__name__)


class TestTimer(ClockAppTestCase):
    """Tests the timer page features"""

    """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""
    def setUp(self):
        super(TestTimer, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        # Move to Timer tab
        self.main_view.switch_to_tab("TimerTab")

    def select_name_preset_label(self):
        label = self.main_view.get_timer_name_preset()
        timeout = 0
        while not label.focus and timeout < 10:
            self.pointing_device.click_object(label)
            sleep(1)
            timeout += 1
        return label

    def delete_first_preset(self):
        self.main_view.drag_page_up("TimerPage", self)

        # Delete the preset
        first_preset = self.main_view.get_first_preset_list_item()
        first_preset.swipe_to_delete()
        first_preset.confirm_removal()

    def add_preset(self, setTime=0):
        # Show the toolbar
        self.main_view.open_toolbar()

        # Click the add preset button
        self.main_view.get_toolbar().click_button('addPresetButton')

        # Set Timer label as "test"
        self.main_view.drag_page_up("TimerPage", self)
        label = self.select_name_preset_label()
        self.assertThat(label.focus, Eventually(Equals(True)))
        self.keyboard.type("test")
        self.assertThat(label.text, Eventually(Equals("test")))

        self.main_view.drag_page_down("TimerPage", self)

        if setTime:
            # Set timer minute
            logger.debug("Grabbing minute hand")
            minute = self.main_view.get_timer_minute_hand()
            timeout = 0

            #need to select clock in order to set time
            while label.focus and timeout < 10:
                logger.debug("Timerlabel loop")
                TimerLabel = self.main_view.get_label_timer()
                self.pointing_device.click_object(TimerLabel)
                sleep(1)
                timeout += 1

            #make sure timer is set
            x, y, w, h = minute.globalRect
            tx = x + (w / 2)
            ty = y + (h / 2.5)
            timeout = 0
            reset_button = self.main_view.get_timer_reset_button()
            logger.debug("reset_button %s" % reset_button.visible)

            #Try again if we failed at first
            while not reset_button.visible and timeout < 10:
                logger.debug("Dragging minute hand")
                self.pointing_device.drag(tx, ty - (h / 4), tx + (w / 2), ty + (h / 2))
                sleep(1)
                timeout += 1

        self.main_view.get_toolbar().click_button('savePresetButton')

    def test_add_preset(self):
        """Test to check if button to add a preset working"""
        num_of_presets_old = self.main_view.get_num_of_presets()
        self.add_preset(1)

        # check that the timer preset has been added.
        self.assertThat(
            self.main_view.get_num_of_presets,
            Eventually(Equals(num_of_presets_old + 1)))

    def test_run_preset(self):
        """Tests if a preset can runs"""

        # Create a new preset
        self.add_preset()

        self.main_view.drag_page_up("TimerPage", self)

        # Click to select the preset
        # FIXME: Instead of always running the first preset, run the preset which was created
        first_preset = self.main_view.get_first_preset_list_item()
        self.pointing_device.click_object(first_preset)

        self.main_view.drag_page_down("TimerPage", self)

        # Click to start the preset
        label = self.main_view.get_label_timer()
        self.pointing_device.click_object(label)

        # Check if timer is started
        self.assertThat(label.text, Eventually(NotEquals("03:00")))

    def test_delete_preset(self):
        """Test if the swipe of a preset deletes it"""

        # Create a new preset
        self.add_preset()

        num_of_presets_old = self.main_view.get_num_of_presets()

        #delete preset
        self.delete_first_preset()

        # Check that the timer preset has been deleted
        self.assertThat(
            self.main_view.get_num_of_presets,
            Eventually(Equals(num_of_presets_old - 1)))

