/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtGraphicalEffects 1.0
import "../common"

ClockBase {
    id: timerFace

    // Properties to store/set the time variables
    property var startTime: 0;
    property var remTime: 0;
    property int totalTime: 0;
    property int pauseTime: 0;

    // Property to activate/deactivate the timer
    property bool timerOn: false;
    property bool inProgressFlag

    // Property to expose the timer visual hint
    property alias timerHintImage: _timerHintImage.source

    showHourHand: false
    draggable: !inProgressFlag

    // Timer function called by the main clock loop
    function onTimerUpdate () {
        if( timerOn ) {
            inProgressFlag = true;
            remTime = totalTime - Math.floor((new Date() - startTime)/ 1000) - pauseTime;

            if (remTime < 0) {
                timerOn = false
                totalTime = -1;
            } else {
                ssToTime(remTime);
            }
        }
    }

    // Function to convert time (in seconds) to hh:mm:ss format
    function ssToTime(time) {
        seconds = time % 60
        time = Math.floor(time / 60)
        minutes = time % 60
    }

    // This timer to required to avoid rare occurences of the timer hand being stuck at the place where it was stopped
    // when the reset button is pressed while the timer is running.
    Timer {
        id: reset_timer
        interval: 200
        repeat: false
        onTriggered: {
            inProgressFlag = true;
            inProgressFlag = false;
        }
    }

    function reset() {
        reset_timer.start()
        timerOn = false;
        state = "";
        startTime = remTime = pauseTime = 0;
        minutes = seconds = totalTime = 0;
    }

    // Image to display the side bars to differentiate it from other tabs
    Image {
        id: _timerHintImage
        smooth: true
        width: units.gu(40)
        anchors.centerIn: parent
        fillMode: Image.PreserveAspectFit
        source: Qt.resolvedUrl("../images/timer_black.png")
    }

    Repeater {
        model: 12
        delegate: AnalogClockMarker {
            number: index
            size: 0.5
            distance: 0.48
        }
    }
}
